<?php

function XML_Template_process_file($filename, $obj, $clean = 1)
{
	$doc = new DOMDocument;
	$doc->load($filename);
	XML_Template_process($doc, $obj, $clean);
	return $doc;
}

function XML_Template_process($node, $obj, $clean = 1)
{
	if (is_a($obj, 'DOMNode')) {                          # overwrite
		for ($i = 0; $i < $node->childNodes->length; ++$i) {
			$child = $node->childNodes->item($i);
			$node->removeChild($child);
		}

		if (is_a($obj, 'DOMDocument')) {
			$obj = $obj->document_element();
		}

		$newobj = $obj->cloneNode(true);
		$node->appendChild($newobj);

		XML_Template_process($newobj, array(), $clean);
	} else if (!is_array($obj)) {                         # overwrite
		for ($i = 0; $i < $node->childNodes->length; ++$i) {
			$child = $node->childNodes->item($i);
			$node->removeChild($child);
		}
		$doc = $node->ownerDocument;
		$node->appendChild($doc->createTextNode($obj));
	} else if (is_associative_array($obj)) {              # substitute
		for ($i = 0; $i < $node->childNodes->length; ++$i) {
			$child = $node->childNodes->item($i);
			$processed = false;

			if ($child->nodeType == XML_ELEMENT_NODE) {
				unset($id);

				$tag = $child->localName;
				$attrs = $child->attributes;

				if (isset($attrs)) {
					foreach ($child->attributes as $attr) {
						if ($attr->namespaceURI == 'http://template.sesse.net/' && $attr->name == 'id') {
							$id = $attr->value;
							if ($clean) {
								$child->removeAttributeNode($attr);
							}
						}
					}
				}
			
				# check all substitutions to see if we found anything
				# appropriate
				foreach (array_keys($obj) as $key) {
					# FIXME: we would want something like \Q and \E here...
					if (preg_match('/^' . $tag . '\/(.*)$/', $key, $matches) ||
					    (isset($id) && preg_match('/^#' . $id . '\/(.*)$/', $key, $matches))) {
					    	$child->setAttribute($matches[1], $obj[$key]);
					}

					if ($processed) {
						continue;
					}
					if ($key == $tag || (isset($id) && $key == ('#'.$id))) {
						XML_Template_process($child, $obj[$key], $clean);
						$processed = true;
					}
				}
			}

			if (!$processed) {
				XML_Template_process($child, $obj, $clean);
			}
		}
	} else {						# repeat
		$doc = $node->ownerDocument;
		$frag = $doc->createElement("temporary-fragment");    # ugh

		while ($node->childNodes->length > 0) {
			$child = $node->childNodes->item(0);
			$frag->appendChild($child);
		}

		foreach ($obj as $instance) {
			if (!isset($instance)) {
				continue;
			}

			$newnode = $frag->cloneNode(true);
			$node->appendChild($newnode);
			XML_Template_process($newnode, $instance, $clean);
			if ($clean) {
				XML_Template_clean($newnode);
			}
		}

		# remove all the <fragment> tags
		for ($i = 0; $i < $node->childNodes->length; ++$i) {
			$child = $node->childNodes->item($i);
			if ($child->localName != 'temporary-fragment') {
				continue;
			}
			while ($child->childNodes->length > 0) {
				$child2 = $child->childNodes->item(0);
				$node->appendChild($child2);
			}
			--$i;
			$node->removeChild($child);
		}	
	}

	if ($clean) {
		XML_Template_clean($node);
	}
}

function XML_Template_clean($node)
{
	if ($node->nodeType == XML_ELEMENT_NODE) {
		if ($node->namespaceURI != 'http://template.sesse.net/') {
			return;
		}

		# as this is a dummy node, we want to remove it and move everything further up
		# after we've done any required replacements
		$doc = $node->ownerDocument;
		$parent = $node->parentNode;

		while ($node->childNodes->length > 0) {
			$child = $node->childNodes->item(0);
			$node->removeChild($child);
			$parent->insertBefore($child, $node);
		}
		$parent->removeChild($node);
	}
}

# FIXME: use varargs here
function XML_Template_alternate($tag, $array, $elems)
{
	$num = count($elems);

	for ($i = 0, $j = 0; $i < count($array); $i++) {
		if (isset($array[$i])) {
			$array[$i][$tag] = $elems[$j++ % $num];
		}
	}

	return $array;
}
		
function is_associative_array($arr)
{
	if (!is_array($arr)) {
		return false;
	}
	$diff = array_diff(range(0, count($arr) - 1), array_keys($arr));
	return (count($diff) > 0);
}
?>
